@@ -12,9 +12,7 @@ import numpy as np
 @hydra.main(config_path="../conf", config_name="main")
 def main(base_config: MOVEConfig): 
     
-    # Merging the user defined data.yaml, model.yaml and training_association.yaml 
-    # with the base_config to override it.
-    print('Overriding the default config with configs from data.yaml, model.yaml and training_association.yaml')
+    # Overriding base_config with the user defined configs.
     cfg = merge_configs(base_config=base_config, 
                         config_types=['data', 'model', 'training_association'])
     
